from ._apis import BaseApi, TournamentApi, EntrantApi, EventApi

class StartGGAPI:
    def __init__(self, api_key):
        if not api_key:
            raise ValueError("api_key must be set.")
        self._base_api = BaseApi(api_key)
        self._tournament = TournamentApi(self._base_api)
        self._entrant = EntrantApi(self._base_api)
        self._event = EventApi(self._base_api)

    @property
    def tournament(self):
        return self._tournament

    @property
    def event(self):
        return self._event

    @property
    def entrant(self):
        return self._entrant



import json

from .QueryStrings import event_entrants_query, event_details_query, sets_query_by_event_id

class EventApi:
    """
    This class wraps the Event query
    """

    def __init__(self, base_api):
        """
        Initializes a new EventApi which uses the base api
        :param BaseApi base_api: the root API object for making all requests.
        """
        self._base = base_api

    def get_event_details(
            self,
            event_id: int,
    ):
        """
        Get all available details for an event

        :returns: dict
        """
        data = {
            "variables": {
                "eventId": str(event_id)
            },
            "query": event_details_query
        }
        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        return json.loads(response.content)

    def find_all_entrants(
        self,
        event_id: int,
        page: int=None,
        per_page: int=None,
        ):
        """
        This function returns a list of entrants for a given event
        :param int event_id:             start.gg eventId
        :param int page:                 page number to pull
        :param int per_page:             page limit
        """
        entrant_list = []
        data = {
            "variables": {
                "perPage": 50,
                "page": 1,
                "eventId": str(event_id)
            },
            "query": event_entrants_query
        }
        if per_page:
            data["variables"]["per_page"] = per_page
        if page:
            data["variables"]["page"] = page

        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        response_json = json.loads(response.content)
        entrant_list += response_json["data"]["event"]["entrants"]["nodes"]

        current_page = 1
        if response_json['data']['event']['entrants']['pageInfo']['totalPages'] > 1:
            while current_page != response_json['data']['event']['entrants']['pageInfo']['totalPages']:
                current_page += 1
                data["variables"]["page"] = current_page
                response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
                entrant_list += json.loads(response.content)["data"]["event"]["entrants"]["nodes"]

        # TODO: properly handle errors from request / expose bad request details, etc
        #return json.loads(response.content)["data"]["event"]["entrants"]["nodes"]
        return entrant_list

    def fetch_sets(
        self,
        event_id,
        ):
        """
        Returns a list of Sets for the event_id
        :param event_id
        """
        set_list = []
        data = {
            "variables": {
                "eventId": str(event_id)
            },
            "query": sets_query_by_event_id
        }
        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        response_json = json.loads(response.content)
        return response_json["data"]["event"]["sets"]["nodes"]



import requests

class BaseApi():
    def __init__(self, api_key):
        self._api_key = api_key

    @property
    def api_key(self):
        return self._api_key

    def raw_request(
            self,
            url: str,
            query_data: dict
    ) -> requests.models.Response:
        response = requests.post(
            url,
            json=query_data,
            headers= {
                "Content-Type": "application/json",
                "Authorization": "Bearer " + self._api_key
            }
        )
        return response



import json

from .QueryStrings import event_entrants_query, event_base_query

# type Entrant {
    # id: ID
    # event: Event
    # initialSeedNum: Int
    # isDisqualified: Boolean
    # name: String (The entrant name as it appears in bracket: gamerTag of the participant or team name)
    # # Paginated sets for this entrant
    # # Arguments
    # # page: [Not documented]
    # # perPage: [Not documented]
    # # sortType: How to sort these sets
    # # filters: Supported filter options to filter down set results.
    # paginatedSets(
    #   page: Int,
    #   perPage: Int,
    #   sortType: SetSortType,
    #   filters: SetFilters
    # ): SetConnection
    # participants: [Participant]
    # seeds: [Seed]
    # skill: Int
    # standing: Standing (Standing for this entrant given an event. All entrants queried must be in the same event (for now))
    # stream: Streams @deprecated( reason: "DEPRECATED. Use streams instead, which supports multiple stream types and teams." )
    # streams: [Streams]
    # team: Team (Team linked to this entrant, if one exists)
# }
class EntrantApi:
    """
    This class wraps the Entrant query
    """

    def __init__(self, base_api):
        """
        Initializes a new EntrantApi which uses the base api
        :param BaseApi base_api: the root API object for making all requests.
        """
        self._base = base_api

    def find_by_event_id(
        self,
        event_id: int,
        page: int=None,
        per_page: int=None,
        ) -> dict:
        """
        This function returns a list of tournaments by a given location
        :param int event_id:             start.gg eventId
        :param int page:                 page number to pull
        :param int per_page:             page limit
        """
        eb = event_base_query.replace("QUERY_PARAMS", "$eventId: ID!, $page: Int!, $perPage: Int!")
        eb = eb.replace("SEARCH_BY", "id: $eventId")
        eb = eb.replace("RESPONSE_KEYS", """
            id
            name
            entrants(query: {
                page: $page
                perPage: $perPage
            }) {
                pageInfo {
                    total
                    totalPages
                }
                nodes {
                    id
                    name
                    initialSeedNum
                }
            }
        """)
        data = {
            "variables": {
                "perPage": 50,
                "page": 1,
                "eventId": str(event_id)
            },
            "query": eb
        }
        if per_page:
            data["variables"]["per_page"] = per_page
        if page:
            data["variables"]["page"] = page

        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        return json.loads(response.content)

    def find_all_by_event_id(self, event_id, per_page=None):
        default_page_size = 50
        all_entrants = []
        if per_page:
            default_page_size = per_page
        response = self.find_by_event_id(event_id, 1, default_page_size)
        all_entrants += response["data"]["event"]["entrants"]["nodes"]
        total_pages = response["data"]["event"]["entrants"]["pageInfo"]["totalPages"]
        for i in range(total_pages+1):
            if i == 0 or i == 1:
                continue
            response = self.find_by_event_id(event_id, i, default_page_size)
            all_entrants += response["data"]["event"]["entrants"]["nodes"]

        return all_entrants



from .BaseApi import BaseApi
from .TournamentApi import TournamentApi
from .EntrantApi import EntrantApi
from .EventApi import EventApi

__all__ = [
    "TournamentApi",
    "EntrantApi",
    "EventApi"
]



import json, time, datetime
from .QueryStrings import query_by_distance, query_by_distance_and_time, tournament_query_by_event_id

class TournamentApi:
    """
    This class wraps the Tournament object query
    https://developer.start.gg/reference/tournament.doc.html
    """

    def __init__(self, base_api):
        """
        Initializes a new TournamentApi which uses the base api
        :param BaseApi base_api: the root API object for making all requests.
        """
        self._base = base_api

    def find_events_by_tournament_slug(
            self,
            tourney_slug: str
    ):
        """
        FUNCTION INFO HERE
        """
        data = {
            "variables": {
                "slug": tourney_slug
            },
            "query": tournament_query_by_event_id
        }
        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        return json.loads(response.content)["data"]["tournament"]["events"]

    def find_by_coords(
            self,
            coords: str,
            before_date=None,
            after_date=None,
            page=None,
            per_page=None,
            radius=None
    ):
        """
        This function returns a list of tournaments by a given location
        :param string coords:            Lat,Lng string
        :param int before_date:          epoch timestamp
        :param int after_date:           epoch timestamp
        :param int page:                 page number to pull
        :param int per_page:             page limit
        :param string radius:            50mi / 50km
        """
        data = {
            "variables": {
                "perPage": 5,
                "page": 1,
                "coordinates": coords,
                "radius": "50mi"
            }
        }
        if per_page:
            data["variables"]["per_page"] = per_page
        if page:
            data["variables"]["page"] = page
        if radius:
            data["variables"]["radius"] = radius

        lookup_query = query_by_distance
        if before_date and after_date:
            lookup_query = query_by_distance_and_time
            data["variables"]["beforeDate"] = round(time.mktime(before_date.timetuple()) + before_date.microsecond/1e6)
            data["variables"]["afterDate"] = round(time.mktime(after_date.timetuple()) + after_date.microsecond/1e6)
        data["query"] = lookup_query

        response = self._base.raw_request("https://api.start.gg/gql/alpha", data)
        return json.loads(response.content)

    def find_all_by_coords(self, coords, before_date=None, after_date=None, per_page=None, radius=None):
        default_page_size = 50
        before_date = datetime.datetime.now() - datetime.timedelta(days=1)
        after_date = datetime.datetime.now() - datetime.timedelta(days=7)

        all_tournaments = []
        if per_page:
            default_page_size = per_page
        response = self.find_by_coords(coords, before_date=before_date, after_date=after_date, page=1, per_page=default_page_size, radius=radius)
        all_tournaments += response["data"]["tournaments"]["nodes"]
        if "pageInfo" in response["data"]["tournaments"]:
            total_pages = response["data"]["tournaments"]["pageInfo"]["totalPages"]

        return all_tournaments



sets_query_by_event_id = """
  query FindSets($eventId: ID!) {
    event(id: $eventId) {
      sets(
        page: 1,
        perPage: 500,
        sortType: STANDARD
      ) {
        nodes {
          round
          fullRoundText
          slots {
            entrant {
              name
            }
          }
        }
      }
    }
  }

"""

tournament_query_by_event_id = """
    query FindEventId($slug: String!) {
        tournament(slug: $slug) {
            id
            name
            events {
                id
                name
            }
        }
    }
"""
tournaments_base_query = """
  query Tournaments(QUERY_PARAMS) {
    event(SEARCH_BY) {
      RESPONSE_KEYS
    }
  }
"""

query_by_distance = """
  query Tournaments($page: Int!, $perPage: Int, $coordinates: String!, $radius: String!) {
    tournaments(query: {
      perPage: $perPage
      page: $page
      filter: {
        location: {
          distanceFrom: $coordinates,
          distance: $radius
        }
      }
    }) {
      pageInfo {
          total
          totalPages
      }
      nodes {
        id
        name
        city
        numAttendees
        slug
        createdAt
        endAt
        images { id }
        links { facebook discord }
      }
    }
  }"""

query_by_distance_and_time = """
  query Tournaments($perPage: Int, $coordinates: String!, $radius: String!, $beforeDate: Timestamp!, $afterDate: Timestamp!) {
    tournaments(query: {
      perPage: $perPage
      filter: {
        location: {
          distanceFrom: $coordinates,
          distance: $radius
        },
        beforeDate: $beforeDate,
        afterDate: $afterDate
      }
    }) {
      nodes {
        id
        name
        city
        numAttendees
        slug
        endAt
      }
    }
  }
"""

base_query_tournaments = """
  query Tournaments(TOP_PARAMS) {
    tournaments(query: QUERY) {
      NODE_DEFINITION
    }
  }"""

event_base_query = """
  query EventEntrants(QUERY_PARAMS) {
    event(SEARCH_BY) {
      RESPONSE_KEYS
    }
  }
"""
event_entrants_query = """
  query EventEntrants($eventId: ID!, $page: Int!, $perPage: Int!) {
    event(id: $eventId) {
      id
      name
      entrants(query: {
        page: $page
        perPage: $perPage
      }) {
        pageInfo {
          total
          totalPages
        }
        nodes {
          id
          participants {
            id
            gamerTag
            user {
              authorizations {
                type
                externalUsername
              }
            }
          }
        }
      }
    }
  }
"""

event_details_query = """
    query EventDetails($eventId: ID!) {
        event(id: $eventId) {
            id
            name
            competitionTier
            isOnline
            numEntrants
            prizingInfo
            rulesMarkdown
            rulesetId
            slug
            startAt
            state
        }
    }
"""
